/*
 ** key.c
 ** 
 ** Made by (Stephane RAULT)
 ** Login   <rault_s@epita.fr>
 ** 
 ** Started on  Fri Apr  9 21:29:44 1999 Stephane RAULT
 ** Last update Tue May 11 00:13:22 1999 Poindessous Thomas
 */

#include "epiwm.h"
#include <ctype.h>

void			handle_keyboard()
{
  char			hit[3];
  KeySym		key;
  XComposeStatus	compose;
  t_shortcut		*shortcut;
  
  XLookupString(&gl_event.xkey, hit, 3, &key, &compose);
  shortcut = (t_shortcut *)malloc(sizeof(t_shortcut));
  if (gl_event.xkey.state & ControlMask)
    shortcut->control = 1;
  else
    shortcut->control = 0;
  if (gl_event.xkey.state & Mod1Mask)
    shortcut->alt = 1;
  else
    shortcut->alt = 0;
  if (gl_event.xkey.state & ShiftMask)
    shortcut->shift = 1;
  else
    shortcut->shift = 0;
  shortcut->key = key;
  compare_key_to_list(shortcut);
  free (shortcut);
}

char		*read_modifier_keys(line, index)
     char	*line;
     int	index;
{
  char		*tokens[4];
  static char	tmpstring[100];
  int		i;

  for (i = 0; i < 4; i++)
    tokens[i] = NULL;
  i = 0;
  strncpy(tmpstring, line, 100);
  (char *) tokens[i] = (char *) strtok(tmpstring, "+");
  while (((char *) tokens[++i] = (char *) strtok(NULL, "+")));
  tokens[i] = NULL;
  return (tokens[index]);
}

int		get_key_name(str, key)
     char	*str;
     t_shortcut	*key;
{
  key->key = XStringToKeysym(str);
  if (key->key == NoSymbol)
    return (0);
  else
    return (1);
}

void		shortcut_list_add(shortcut, command, p_func)
     t_shortcut	*shortcut;
     char	*command;
     void	(*p_func)();
{
  t_shortcut_list *tmp;
  t_shortcut_list *list;
  
  tmp = (t_shortcut_list *)malloc(sizeof(t_shortcut_list));
  list = (t_shortcut_list *)malloc(sizeof(t_shortcut_list));
  tmp->shortcut = shortcut;
  tmp->cmd = NULL;
  tmp->p_func = NULL;
  if (command)
    {
      tmp->cmd = (char *)malloc(sizeof(char)*(strlen(command)+1));
      strcpy(tmp->cmd, command);
    }
  else if (p_func)
    tmp->p_func = p_func;
  tmp->next = NULL;
  if (key_list == NULL)
    key_list = tmp;
  else
    {
      list = key_list;
      while (list->next)
	list = list->next;
      list->next = tmp;
    }
}

t_shortcut	*create_shortcut(control, alt, shift, key)
     char	control;
     char	alt;
     char	shift;
     KeySym	key;
{
  t_shortcut	*shortcut;
  
  shortcut = (t_shortcut *)malloc(sizeof(t_shortcut));
  shortcut->control = control;
  shortcut->alt = alt;
  shortcut->shift = shift;
  shortcut->key = key;
  return (shortcut);
}

void print_left()
{
/*  printf("Left\n");*/
}

void print_right()
{
/*   printf("Right\n");*/
}

void print_up()
{
/*   printf("Up\n");*/
}

void print_down()
{
/*   printf("Down\n");*/
}

void			compare_key_to_list(shortcut)
t_shortcut		*shortcut;
{
   t_shortcut_list	*list;
   
   list = key_list;
   while (list)
     {
	if ((list->shortcut->control == shortcut->control) &&
	    (list->shortcut->alt == shortcut->alt) &&
	  (list->shortcut->shift == shortcut->shift) &&
	  (list->shortcut->key == shortcut->key))
	{
	  if (list->cmd)
	    system(list->cmd);
	  else if (list->p_func)
	    list->p_func();
	}
      list = list->next;
    }
}

void		key_build(file)
     FILE		*file;
{
  char		tmp[250];
  char		*key;
  char		*key_comb;
  char		*key_return;
  char		ctrl;
  char		alt;
  char		shift;
  t_shortcut	shortcut;
 
  while (fgets(tmp, 300, file) != NULL)
    {
      if ((*tmp != '#') && (*tmp != '\t') && (*tmp != '\n') && (*tmp != ' '))
	{
	  ctrl = 0;
	  alt = 0;
	  shift = 0;
	  key = NULL;
	  if (strlen(tmp) != 1)
	    tmp[strlen(tmp) - 1] = ' ';
	  key_comb = index_file(tmp, 0);
	  if ((key_return = read_modifier_keys(key_comb, 0)))
	    {
	      if (!strcmp("Control", key_return))
		ctrl = 1;
	      else if (!strcmp("Alt", key_return))
		alt = 1;
	      else if (!strcmp("Shift", key_return))
		shift = 1;
	      else if (!key)
		key = index_file(key_return, 0);
	    }
	  if ((key_return = read_modifier_keys(key_comb, 1)))
	    {
	      if (!strcmp("Control", key_return))
		ctrl = 1;
	      else if (!strcmp("Alt", key_return))
		alt = 1;
	      else if (!strcmp("Shift", key_return))
		shift = 1;
	      else if (!key)
		key = index_file(key_return, 0);
	    }
	  if ((key_return = read_modifier_keys(key_comb, 2)))
	    {
	      if (!strcmp("Control", key_return))
		ctrl = 1;
	      else if (!strcmp("Alt", key_return))
		alt = 1;
	      else if (!strcmp("Shift", key_return))
		shift = 1;
	      else if (!key)
		key = index_file(key_return, 0);
	    }
	  if ((key_return = read_modifier_keys(key_comb, 3)))
	    {
	      if (!key)
		key = index_file(key_return, 0);
	    }
	  if (!get_key_name(key, &shortcut))
	    {
	      fprintf(stderr, "bad shortcut key : %s\n", key);
	      exit (1);
	    }
	  if (!strcmp(index_file(tmp, 1), "Exec"))
	    shortcut_list_add(create_shortcut(ctrl, alt, shift, shortcut.key),
			      extract_str(tmp, "Exec "), NULL);
	  else if (!strcmp(index_file(tmp, 1), "Fonc"))
	    {
	      shortcut_list_add(create_shortcut(ctrl, alt, shift, 
						shortcut.key), NULL, NULL);
	      
	    }
	  else
	    {
	      fprintf(stderr, "error in key reading : %s\n", 
		      index_file(tmp, 1));
	      exit (1);
	    }
	}
      
    }
}




